<?php
/**
 * GMapFactory configuration file
 *
 * @project     GMapFactory
 * @revision    $Id: globprefs.php,v 1.18.2.1 2004/07/15 19:34:56 zak Exp $
 * @purpose     This file contains the global configurations for GMapFactory.
 * @author      William A. Bronsema, C.E.T. (bronsema@dmsolutions.ca)
 * @copyright
 * <b>Copyright (c) 2002, DM Solutions Group Inc.</b>
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
// define the path to the conguration settings
define( "CONFIG_PATH", "../../config/" );

// define the path to the components folder
define( "COMMON", "../common/" );

// include the logging functions
include_once( COMMON."logger/logfile.php" );
include_once( COMMON."logger/error_manager.php" );
include_once( COMMON."logger/logger.php" );

// include the xml parsing class
include_once( COMMON."appcontext/appcontext.php" );
include_once( COMMON."session/session.php" );
installSessionDirectoryHandler();
initializeSession();

$gbVoidSession = false;

// check if the IP has been set and validate
if ( isset( $_SESSION["gszRemoteAdd"] ) && 
                                strlen(trim($_SESSION["gszRemoteAdd"])) > 0 )
{
    // check if IP matches current client
    if ( trim( $_SESSION["gszRemoteAdd"] ) != trim( $_SERVER["REMOTE_ADDR"] ) )
    {
        // possible security breach void session
        $gbVoidSession = true;
    }
}
else
{
    // possible security breach void session
    $gbVoidSession = true;
}

// define a new AppContext object
$gContext = new AppContext( CONFIG_PATH."gmapfactory.xml" );

//read context values out of current context object
// main application directory
$gszAppPath = str_replace("\\", "/", realpath($gContext->getContextValue("app_path")));
if (substr($gszAppPath, -1) != "/")
    $gszAppPath .= "/";

// the new applications base path
$gszNewAppPath = str_replace("\\", "/", realpath($gContext->getContextValue("new_app_path")));
if (substr($gszNewAppPath, -1) != "/")
    $gszNewAppPath .= "/";

// the new applications base url
$gszNewAppURL = str_replace("\\", "/", $gContext->getContextValue("new_app_url"));
if (substr($gszNewAppURL, -1) != "/")
    $gszNewAppURL .= "/";

// the mapscript module
$gszMapscriptModule = $gContext->getContextValue("mapscript_module");

// image path and url
$gszTmpImgPath = str_replace("\\", "/", realpath($gContext->getContextValue("tmp_img_path")));
if (substr($gszTmpImgPath, -1) != "/")
    $gszTmpImgPath .= "/";
if (!isset($_SESSION["gszTmpImgPath"]))
    $_SESSION["gszTmpImgPath"] = $gszTmpImgPath;
$gszTmpWebImgPath = str_replace("\\", "/", realpath($gContext->getContextValue("tmp_web_img_path")));
if (substr($gszTmpWebImgPath, -1) != "/")
    $gszTmpWebImgPath .= "/";
if (!isset($_SESSION["gszTmpWebImgPath"]))
    $_SESSION["gszTmpWebImgPath"] = $gszTmpWebImgPath;

// path for the file browser root directory
$gszFileBrowserRoot = str_replace("\\", "/", realpath($gContext->getContextValue("file_browser_root")));
if (substr($gszFileBrowserRoot, -1) != "/")
    $gszFileBrowserRoot .= "/";

$_SESSION['gszFileBrowserRoot'] = $gszFileBrowserRoot;

// path for the log file
$gszLogPath = str_replace("\\", "/", realpath($gContext->getContextValue("log_path")));
if (substr($gszLogPath, -1) != "/")
    $gszLogPath .= "/";

/// Group name that gmapfactory should use for newly created file to.
$gszGroupName = $gContext->getContextValue("group_name");

// get the log level
$gszLogLevel = $gContext->getContextValue("log_level");

// the app and project folder names
$gszGMFAppFolderName =  $gContext->getContextValue("app_folder_name");
$gszGMFProjFolderName =  $gContext->getContextValue("proj_folder_name");

// navigation mode default (either ROSA or HTML)
$gszDefaultMode = $gContext->getContextValue("default_navigation_mode");

// define the templates directory
define( "TEMPLATES", $gszAppPath."htdocs/gmapfactory/templates/" );

// create new logfile object (customize parameters)
$oLogFile = new LogFile( $gszLogPath."gmapfactory.log", LOG_TO_FILE, 1 );

// set the max log level ( set in gmapfactory.xml )
switch( $gszLogLevel )
{
    case "LOG_ALL";
        $gnLogLevel = LOG_ALL;
        break;
    case "LOG_VERBOSE":
        $gnLogLevel = LOG_VERBOSE;
        break;
    case "LOG_QUIET":
        $gnLogLevel = LOG_QUIET;
        break;
    default:
        $gnLogLevel = LOG_OFF;
} // switch

$oLogFile->setMaxLogLevel( $gnLogLevel );

// create new error manager object
$oErrorManager = new ErrorManager();

// create a logger object
$oLog = new Logger( "General" );

// set the global log file and error manager
$oLog->setLogFile( $oLogFile );
$oLog->setErrorManager( $oErrorManager );

// convert the global session reference if necessary
if ( PHP_OS == "WINNT" || PHP_OS == "WIN32" )
{
    if ( version_compare( phpversion(), "4.2.1" ) < 0 )
        $_SESSION = &$HTTP_SESSION_VARS;
}

$szURI = "http://".$_SERVER['HTTP_HOST'].
         dirname($_SERVER['PHP_SELF']);


?>
