<?php

/**
 * globprefs.php sets up the correct environment for all mapedit components.
 *
 * @project     MapEdit
 * @revision    $Id: globprefs.php,v 1.66.2.1 2003/10/15 20:04:57 zak Exp $
 * @purpose     global environment for mapedit
 * @author      Paul Spencer (spencer@dmsolutions.ca)
 * @copyright
 * <b>Copyright (c) 2002, DM Solutions Group Inc.</b>
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

//the PHP Utils are stored in a common directory that is not necessarily
//rooted on the directory that this file is stored in.  In order to make
//all the includes work correctly, we must define COMMON to be the relative
//path from this directory to the directory under which all the PHP Utils
//are stored.  The tag COMMON is used in the PHP Utils to determine how
//to find other PHP Utils.
define( "COMMON", "../common/" );

//the location of the configuration files
define( "CONFIG_PATH", "../../config/" );

/// MapEdit execution mode (0 -> DEMO, 1 -> NORMAL, 2 -> EXPERT)
define ( "DEMO_MODE", 0 );
define ( "NORMAL_MODE", 1 );
define ( "EXPERT_MODE", 2 );

//define min and max modes for error checking.
define ( "MIN_MODE", DEMO_MODE );
define ( "MAX_MODE", EXPERT_MODE );

//include logger, log file, error manager and appcontext from PHP Utils
include_once( COMMON."logger/logger.php" );
include_once( COMMON."logger/logfile.php" );
include_once( COMMON."logger/error_manager.php" );
include_once( COMMON."appcontext/appcontext.php" );

//mapeditapp class definition
include_once( "mapeditapp.php" );

include_once( COMMON."session/session.php" );
installSessionDirectoryHandler();
initializeSession();

$gbVoidSession = false;

//create global logging before the session starts so that
//session preserved objects have something to connect to
$gLogFile = new LogFile( "/tmp/mapedit.log", LOG_TO_FILE, true );

// convert the global session reference if necessary
if ( PHP_OS == "WINNT" || PHP_OS == "WIN32" )
{
    if ( version_compare( phpversion(), "4.2.1" ) < 0 )
        $_SESSION = &$HTTP_SESSION_VARS;
}

//starting the session resumes everything where we left off at the end of the
//last script
if (!isset($_SESSION["gErrorManager"]))
{
    $_SESSION['gErrorManager'] = new ErrorManager();
}

//create context only once per session
if (!isset($_SESSION["gMapEditContext"]))
{
    $_SESSION["gMapEditContext"] = new AppContext( CONFIG_PATH."mapedit.xml" );
    $_SESSION["gMapEditContextMTime"] = filemtime( CONFIG_PATH."mapedit.xml" );
}

//reload context if changed during a session
if ( $_SESSION["gMapEditContextMTime"] !=
     filemtime( CONFIG_PATH."mapedit.xml" ))
{
    $_SESSION["gMapEditContext"] = new AppContext( CONFIG_PATH."mapedit.xml" );
    $_SESSION["gMapEditContextMTime"] = filemtime( CONFIG_PATH."mapedit.xml" );
}

//read context values out of current context object

/// Start in this directory when opening a map file.
$gszWorkingDir = getContextPath($_SESSION["gMapEditContext"],
                                "file_browser_root");

// Main application directory.
$gszAppPath = dirname($_SERVER["PATH_TRANSLATED"]);

// Where to put log files.
$gszLogPath = getContextPath( $_SESSION["gMapEditContext"], "log_path");

$aszLog = explode(".", basename($_SERVER["PHP_SELF"]));

$gLogFile->LogFile( $gszLogPath.$aszLog[0]."_mapedit.log", LOG_TO_FILE, true );

/// config file
$_SESSION['gszMapservFile'] = CONFIG_PATH.(getContextValue( $_SESSION["gMapEditContext"],
                                                "configuration_file"));

/// Group name that mapedit should use for newly created file to.
$gszGroupName = getContextValue( $_SESSION["gMapEditContext"], "group_name");

/// Get new file permission mask
$gszPermissionMask =
          $_SESSION["gMapEditContext"]->getContextValue("new_file_permission");
if (strlen($gszPermissionMask) != 4 ||
    $gszPermissionMask[0] != "0" || // first digit must be 0
    $gszPermissionMask[1] < 6 ||    // second must be 6 or 7
    $gszPermissionMask[1] > 7 ||    // ...
    $gszPermissionMask[2] < 0 ||    // Last two digits must be
    $gszPermissionMask[2] > 7 ||    // 0 to 7
    $gszPermissionMask[3] < 0 ||    // ...
    $gszPermissionMask[3] > 7)      // ...
  $gszPermissionMask = "0644";

//set the max log level
$nLogLevel=0;
$szTmpLogLevel = $_SESSION["gMapEditContext"]->getContextValue("log_level");
if ( $szTmpLogLevel === false )
{
    // something failed (bad path in XML?)
    $szTmpLogLevel = 0;

    // give error message
    $_SESSION["gErrorManager"]->setError( ERR_CRITICAL,
        "An error occurred accessing the log ".
        "level from the mapedit.xml file.  One possible cause of this ".
        "problem is an invalid path to the maplab.xml.  Please check the ".
        "mapedit.xml configuration file and ensure that the path to the ".
        "maplab.xml file is valid." );
}

eval( "\$nLogLevel = $szTmpLogLevel;" );
$gLogFile->setMaxLogLevel($nLogLevel);

if (!isset($gnExecMode) ||
           $gnExecMode < MIN_MODE ||
           $gnExecMode > MAX_MODE )
{
    $gnExecMode = getContextValue( $_SESSION["gMapEditContext"],
                                   "execution_mode");
}
if ($gnExecMode === false)
    $gnExecMode = NORMAL_MODE;
elseif ($gnExecMode == DEMO_MODE)
{
    // if it is a demo mode, lets force the working_path
    $gszWorkingDir =
                    $_SESSION["gMapEditContext"]->getContextValue("demo_path");
}

$_SESSION['gszWorkingDir'] = $gszWorkingDir;

/*****************************************************************************
 *
 * register other session variables as required
 *
 * This array contains the name of the session variable to register followed
 * by (optionally) the default value OR the function to all and the value to
 * pass to the function.
 * This allows you to specify that the default value for the session variable
 * comes from the context object, for instance.
 *
 *****************************************************************************/
//I'm lazy ;) PS
$oContext = $_SESSION["gMapEditContext"];
$aszSessionVars = array( "gszSerializedAppFile",
                         "gszMap",
                         "gszMapName",
                         "gszIndex",
                         "goObjectToCopy",
                         "gnCurrentLayerIndex",
                         "gszFontFile",
                         "gszSymbolFile",
                         "gszCurrentState",
                         "gszWarehouseMapFiles|getContextValue|\$oContext, \"warehouse_mapfiles\"",
                         "gszPHPMapScriptModName|getContextValue|\$oContext, \"mapscript_module\"",
                         "gszImageType|getContextValue|\$oContext, \"image_type\"",
                         "gszTmpImgPath|getContextPath|\$oContext, \"tmp_img_path\"",
                         "gszTmpWebImgPath|getContextPath|\$oContext, \"tmp_web_img_path\"",
                         "gszTmpMapPath|getContextPath|\$oContext, \"tmp_map_path\"",
                         "gszNavigationMode|getContextValue|\$oContext, \"default_navigation_mode\"");
foreach($aszSessionVars as $mValue)
{
    $amValues = explode( "|", $mValue );
    if ( !isset($_SESSION[$amValues[0]]))
    {
        //$gLogFile->logMessage( LOG_QUIET, "$szSessionVar is not registered");
        if ( count( $amValues ) == 3 )
        {
            eval( "\$mDefault = ".$amValues[1]."(".$amValues[2].");");
        }
        elseif (count( $amValues ) == 2 )
        {
            $mDefault = $amValues[1];
        }
        else
        {
            $mDefault = "";
        }
        $_SESSION[$amValues[0]] = $mDefault;
    }
    else
    {
        $gLogFile->logMessage( LOG_VERBOSE, $amValues[0]." is ".
                               $_SESSION[$amValues[0]] );
    }
}

//create and initialize the main application object
//if there isn't a previous serialized app file, create a new gApp object
//NOTE: call $_SESSION["gszSerializedAppFile"] = serializeObj( $gApp );
//at the end of any script that modifies $gApp
$gLogFile->logMessage( LOG_QUIET, "initializing MapEditApp" );
$gLogFile->logMessage( LOG_QUIET, $_SESSION["gszMapservFile"] );

//this will return 'false' if it doesn't work
$gApp = unserializeObj( "gApp" );

//if the unserialize doesn't work, create a new one.
if ( $gApp === false )
{
    $gLogFile->logMessage( LOG_QUIET, "creating new gApp" );
    $gApp = new MapEditApp( );
    $gApp->initialize( $_SESSION['gszMapservFile'] );
}

//if $gApp->oMap is not valid and gszMap is valid, reparse it
//if gszCurrentState is set, then we have come from MapBrowser and need to
//reparse gszMap if it is valid.
if ((!is_object( $gApp->oMap ) || $_SESSION["gszCurrentState"] != "") && $_SESSION["gszMap"] != "")
{

    $gLogFile->logMessage( LOG_QUIET, "reparsing gszMap." );

    //unserialize failed for some reason
    //save to temp file and run ms_tokenizeMap()
    $szTmpMapFile = $_SESSION["gszTmpMapPath"].uniqid("").".map";
    $fp = fopen( $szTmpMapFile, "w+" );
    if ($fp === false)
    {
        $_SESSION["gErrorManager"]->error( ERR_CRITICAL,
                    "failed to write map file $szTmpMapFile" );
    }
    else
    {
        fwrite( $fp, $_SESSION["gszMap"] );
        fclose( $fp );
        $gApp->parseMapFile( $szTmpMapFile );
        serializeObj( "gApp", $gApp );
        unlink($szTmpMapFile);
    }
}

//absolute url to base directory used to make sure all references
//are correct for Netscape
$szURI = "http://".$_SERVER['HTTP_HOST'].
         dirname($_SERVER['PHP_SELF']);

//set up the globally accessible variables from the current map file if
//appropriate.
$oMap = $gApp->getObjByIndex( "" );
if (is_object($oMap))
{
    $_SESSION["gnCurrentLayerIndex"] =
                 $oMap->getLayerIndex($_SESSION["gszIndex"]);

    $_SESSION["gszFontFile"] = $oMap->maoTypes['FONTSET']->getValue();
    if ( substr($_SESSION["gszFontFile"], 0, 1) == "." )
        $_SESSION["gszFontFile"] = realpath(dirname($_SESSION["gszMapName"]).
                                            "/".$_SESSION["gszFontFile"]);
    $gLogFile->logMessage( LOG_QUIET, "fontset is ".
                                       $_SESSION["gszFontFile"] );
    $_SESSION["gszSymbolFile"] = $oMap->maoTypes['SYMBOLSET']->getValue();
    if ( substr($_SESSION["gszSymbolFile"], 0, 1) == "." )
        $_SESSION["gszSymbolFile"] = realpath(dirname($_SESSION["gszMapName"]).
                                              "/".$_SESSION["gszSymbolFile"]);
    $gLogFile->logMessage( LOG_QUIET, "symbolset is ".
                                       $_SESSION["gszSymbolFile"] );
}
else
{
    //an error will have occurred because no map file is loaded, remove it.
    $_SESSION["gErrorManager"]->popLastError();
}

//clear the MapBrowser navigation state to force a reload from the gszMap
$_SESSION["gszCurrentState"] = "";

//anything else?

/**
 * utility function to get a value out of a context, turns all \ to /
 */
function getContextValue( $oContext, $szObject )
{
    $szResult = str_replace("\\", "/",
                    $oContext->getContextValue($szObject));
    return $szResult;
}

/**
 * utility function to get a path value out of a context, adds the trailing / by default
 * and turns all \ to /
 */
function getContextPath( $oContext, $szObject )
{
    $szResult = str_replace("\\", "/",
                    $oContext->getContextValue($szObject));
    if (substr($szResult, -1) != "/")
        $szResult .= "/";
    return $szResult;
}

/**
 * utility function to deserialize a file to it's object representation,
 * assumes that the file contains a single object
 */
function unserializeObj( $szObjName )
{
    global $gLogFile;

    $mObj = false;
    if (!isset($_SESSION["gaSerializedObjs"][$szObjName]))
    {
        $gLogFile->logMessage( LOG_QUIET, "invalid object ($szObjName)." );
    }
    elseif (!is_file($_SESSION["gaSerializedObjs"][$szObjName]))
    {
        $gLogFile->logMessage( LOG_QUIET, "invalid file ($szObjName)." );
    }
    else
    {
        $fp = @fopen( $_SESSION["gaSerializedObjs"][$szObjName], "r" );
        if ($fp === false)
        {
            $gLogFile->logMessage( LOG_QUIET, "error opening ".
                            $_SESSION["gaSerializedObjs"][$szObjName] );
        }
        else
        {
            $szSerializedObj = fread($fp,
                        filesize($_SESSION["gaSerializedObjs"][$szObjName]));
            $gLogFile->logMessage( LOG_QUIET,
                                "restoring $szObjName from ".
                              $_SESSION["gaSerializedObjs"][$szObjName] );
            $gLogFile->logMessage( LOG_ALL, $szSerializedObj );
            $mObj = unserialize( $szSerializedObj );
            fclose( $fp );
        }
    }
    return $mObj;
}

/**
 * utility function to serialize an object and return a tmp file name that it was
 * serialized in
 *
 * @param szObjName string the name to remember the serialized object by.
 * @param mObj mixed object to serialize
 * @return string the filename that was serialized or false if it failed
 */
function serializeObj( $szObjName, &$oObj )
{
    global $gLogFile;

    $bResult = false;

    //use the same storage path as the session
    $szSessionPath = realpath(getSessionSavePath());

    if (!isset($_SESSION["gaSerializedObjs"]))
    {
        $_SESSION["gaSerializedObjs"] = array();
    }
    else
    {
        //remove the old file if possible
        $bResult = @unlink($_SESSION["gaSerializedObjs"][$szObjName]);
        if ( !$bResult )
        {
            $gLogFile->logMessage( LOG_QUIET, "failed to delete ".
                                   $_SESSION["gaSerializedObjs"][$szObjName] );
        }
    }
    $_SESSION["gaSerializedObjs"][$szObjName] = $szSessionPath."/".
                                                        (uniqid( rand(), 1 ));
    $gLogFile->logMessage( LOG_QUIET, "writing object to ".
                                   $_SESSION["gaSerializedObjs"][$szObjName] );

    //open the file using the created name.
    $fp = @fopen( $_SESSION["gaSerializedObjs"][$szObjName], "w" );

    //if an error occurs opening the file, return false
    if ( $fp === false )
    {
        $gLogFile->logMessage( LOG_QUIET, "error opening ".
                                    $_SESSION["gaSerializedObjs"][$szObjName].
                                    "for writing!" );
        $bResult = false;
    }
    else
    {
        $szRepr = serialize( $oObj );
        $nWriteLen = strlen( $szRepr );
        $nBytes = fwrite( $fp, $szRepr, $nWriteLen );
        if ($nBytes != $nWriteLen)
        {
            $gLogFile->logMessage( LOG_QUIET, "error writing object to ".
                                $_SESSION["gaSerializedObjs"][$szObjName] );
            $bResult = false;
        }
        else
        {
            $gLogFile->logMessage( LOG_QUIET, "wrote $nBytes to ".
                                $_SESSION["gaSerializedObjs"][$szObjName] );
        }
        fclose( $fp );
    }
    $gLogFile->logMessage( LOG_QUIET, "returning $bResult." );

    return $bResult;
}
?>
